"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const treeSitterHast = require("tree-sitter-hast");
const visit = require("unist-util-visit");
const parse_1 = require("./parse");
function isOptions(value) {
    if (!value)
        throw new Error('Missing options');
    if (!!value.grammars && !!value.grammarPackages)
        throw new Error('grammars or grammarPackages must be specified in options');
    return true;
}
const attacher = (options) => {
    if (!isOptions(options))
        throw new Error('Invalid options');
    let loaders = (options.grammarPackages || []).map(pkg => treeSitterHast.loadLanguagesFromPackage(pkg));
    const parserPromise = Promise.all(loaders).then(loadedLanguages => {
        const grammars = Object.assign({}, options.grammars);
        for (const map of loadedLanguages) {
            for (const entry of map.entries()) {
                grammars[entry[0]] = entry[1];
            }
        }
        Object.assign(grammars, options.grammars);
        return new parse_1.MultiLanguageParser(grammars);
    });
    const transformer = (tree, _file) => __awaiter(this, void 0, void 0, function* () {
        const parser = yield parserPromise;
        visit(tree, 'code', node => {
            const lang = node.lang;
            if (lang && parser.canParseLanguage(lang)) {
                const highlighted = treeSitterHast.highlightText(parser.getParser(lang), parser.getScopeMappings(lang), node.value, { classWhitelist: options.classWhitelist });
                if (!node.data) {
                    node.data = {};
                }
                node.data.hProperties = {
                    className: [
                        'tree-sitter',
                        ...(lang ? [`language-${lang}`] : [])
                    ]
                };
                node.data.hChildren = [highlighted];
            }
        });
        return tree;
    });
    return transformer;
};
module.exports = attacher;
