"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Parser = require("tree-sitter");
class MultiLanguageParser {
    constructor(grammars) {
        this.parsers = new Map();
        this.grammars = grammars;
    }
    canParseLanguage(language) {
        return !!this.grammars[language];
    }
    getParser(language) {
        const grammar = this.grammars[language];
        if (!grammar)
            throw new Error('invalid language');
        let parser = this.parsers.get(grammar);
        if (!parser) {
            this.parsers.set(grammar, parser = new Parser());
            parser.setLanguage(grammar.grammar);
        }
        return parser;
    }
    getScopeMappings(language) {
        const grammar = this.grammars[language];
        if (!grammar)
            throw new Error('invalid language');
        return grammar.scopeMappings;
    }
}
exports.MultiLanguageParser = MultiLanguageParser;
